import { normalize, normalizePhone } from '../helpers.js'

/** Regular expression for cellphone numbers. Cellphone numbers start with +417 */
const reMobile = /^\+417/

/** Class representing a player */
class Player {
  /**
   * Create a player
   * A player data item in the Swisstennis PlayerList.xlsx file has the following columns
   * Konkurrenz | Anmeldedatum | Lizenznummer | Klub | Klub Name | Name | Vorname | Geburtsdatum | Adresse | c/o | PLZ | Ort |
   * Land | Tel P | Tel G | Mobile | Email | Klassierung | Klass. Wert | Gesetzte | Kommentar | Einschränkungen | Kommentar |
   * Lizenz Nr.Doppelpartner | Name Doppelpartner | Vorname Doppelpartner | Geburtsdatum Doppelpartner | Klassierung Doppelpartner |
   * Klass. Wert Doppelpartner | bestätigt | On-line Anmeldung | bezahlt
   */
  constructor (data) {
    this.Konkurrenz = normalize(data[0])
    this.Lizenz = normalize(data[2])
    this.geschlecht = null
    if (this.Konkurrenz) {
      if (this.Konkurrenz[0] === 'M') {
        this.geschlecht = 'm'
      } else if (this.Konkurrenz[0] === 'W') {
        this.geschlecht = 'w'
      }
    }
    this.Name = normalize(data[5])
    this.Vorname = normalize(data[6])
    this.Geburtsdatum = data[7]
    this.TelP = normalizePhone(data[13])
    this.TelG = normalizePhone(data[14])
    this.Mobile = normalizePhone(data[15])
    this.Email = normalize(data[16])
    this.Klassierung = normalize(data[17])
    this.LizenzDP = normalize(data[23])
    this.NameDP = normalize(data[24])
    this.VornameDP = normalize(data[25])
    this.GeburtsdatumDP = data[26]
    this.KlassierungDP = normalize(data[27])
    this.Bestaetigt = data[29]
    this.Bezahlt = data[31]
    this.BezahltAm = null
    this.Matches = []
    this.isDoubles = !!this.Konkurrenz.match(/DM.*|[MW]D.*/)
    this.isJunior = (this.Geburtsdatum) ? this.Geburtsdatum.getTime() >= (new Date((new Date()).getFullYear() - 19, 11, 31, 23, 59, 59, 999)).getTime() : false
    this.isJuniorDP = (this.isDoubles && this.GeburtsdatumDP) ? this.GeburtsdatumDP.getTime() >= (new Date((new Date()).getFullYear() - 19, 11, 31, 23, 59, 59, 999)).getTime() : false
    this.name = this.isDoubles ? `${this.Name} ${this.Vorname} / ${this.NameDP} ${this.VornameDP}` : `${this.Name} ${this.Vorname}`
    if (this.Mobile && this.Mobile.match(reMobile)) {
      this.phone = this.Mobile
    } else if (this.TelP && this.TelP.match(reMobile)) {
      this.phone = this.TelP
    } else if (this.TelG && this.TelG.match(reMobile)) {
      this.phone = this.TelG
    } else if (this.Mobile && this.Mobile.match(/FEHLER/)) {
      this.phone = this.Mobile
    } else if (this.TelP && this.TelP.match(/FEHLER/)) {
      this.phone = this.TelP
    } else if (this.TelG && this.TelG.match(/FEHLER/)) {
      this.phone = this.TelG
    }
  }
}

export default { Player }
